#include "ch.hh"

chdiff_type situation_diff(const situation&a, const situation& b)
{
    bool vol_diffs = false;
    
    const double fatal_amplitude = 0.8;
    const double minor_amplitude = 1/4.0;
    
    const double major_octave_diff =
        1.077982183042855; // 2^(1.3/12.0)

    situation::const_iterator A = a.begin();
    situation::const_iterator B = b.begin();
    
    for(;;)
    {
        if(A == a.end())
        {
            for(; B != b.end(); ++B)
            {
                const ch& bch = B->second;
                // b alone
                if(bch.amplitude >= fatal_amplitude) return dramatic_diff;
                if(bch.amplitude >= minor_amplitude) vol_diffs = true;
            }
            break;
        }
        if(B == b.end())
        {
            // a alone
            do {
                const ch& ach = A->second;
                if(ach.amplitude >= fatal_amplitude) return dramatic_diff;
                if(ach.amplitude >= minor_amplitude) vol_diffs = true;
            } while(++A != a.end());
            break;
        }

        const ch& ach = A->second;
        const ch& bch = B->second;
        if(A->first != B->first)
        {
            if(ach.amplitude >= fatal_amplitude) return dramatic_diff;
            if(bch.amplitude >= fatal_amplitude) return dramatic_diff;
            if(ach.amplitude >= minor_amplitude) vol_diffs = true;
            if(bch.amplitude >= minor_amplitude) vol_diffs = true;
            if(A->first < B->first) ++A; else ++B;
            continue;
        }
        
        double octave_diff = ach.hz / bch.hz;
        if(octave_diff < 1) octave_diff = 1.0/octave_diff;
        
        if(octave_diff >= major_octave_diff) return dramatic_diff;
        
        const double ampli_diff = fabs(ach.amplitude - bch.amplitude);
        if(ampli_diff >= fatal_amplitude) return dramatic_diff;
        if(ampli_diff >= minor_amplitude) vol_diffs=true;
        ++A; ++B;
    }
    return vol_diffs ? volume_diff : no_diff;
}
